import typing

import phonenumbers
from email_validator import validate_email


class QiwiCustomer:
    """
    Объект пользователя/покупателя. Неизвестно, зачем эта информация нужна QIWI, но раз можно, то почему бы и нет. Для удобства работы с QIWI API.

    **Аргументы**

    :param phone: номер телефона практически в любом формате
    :type phone: ``str`` or ``int``, optional
    :param email: электронная почта
    :type email: ``str``, optional
    :param account: идентификатор аккаунта
    :type account: ``str``, optional
    :param json_data: словарь с полями phone, email и account. При наличии этого параметра, другие игнорируются.
    :type json_data: ``dict``, optional
    :param ignore_valid: игнорировать невалидные значения номера телефона и почты
    :type ignore_valid: ``bool``, optional
    :param ignore_args: игнорировать отсутствующее значение и поставить None вместо него.
    :type ignore_args: ``bool``, optional
    :rtype: dict

    **Атрибуты**

    :param phone: номер телефона
    :type phone: ``str`` в формате *+79994442211*
    :param email: адрес электронной почты
    :type email: ``str``
    :param account: идентификатор клиента в вашей системе
    :type account: ``str``
    :param dict: json-словарь для использования в API
    :type dict: ``dict``
    """

    def __init__(
        self,
        phone: typing.Union[str, int] = None,
        email: str = None,
        account: typing.Union[str, int] = None,
        json_data: dict = None,
        ignore_valid: bool = False,
        ignore_args: bool = False,
    ):
        if json_data:
            self.phone = json_data["phone"]
            self.email = json_data["email"]
            self.account = json_data["account"]
        else:
            if all([phone, email, account]) or ignore_args:
                if ignore_valid:
                    self.phone = str(phone)
                    self.email = str(email)
                    self.account = str(account)
                else:
                    parse_phone = phonenumbers.parse("+" + str(phone))
                    if phonenumbers.is_valid_number(parse_phone):
                        self.phone = (
                            f"+{parse_phone.country_code}{parse_phone.national_number}"
                        )
                    else:
                        ValueError("Customer phone number is not valid")
                    self.email = validate_email(email).ascii_email
                    self.account = str(account)
            else:
                raise ValueError("One of the values is missing")
        self.dict = {"phone": self.phone, "email": self.email, "account": self.account}



import asyncio
import hashlib
import hmac
import inspect

from hypercorn.asyncio import serve
from hypercorn.config import Config
from starlette import status
from starlette.applications import Starlette
from starlette.requests import Request
from starlette.responses import Response
from starlette.routing import Route

from pyqiwip2p.p2p_types import Bill


class AioQiwiNotify:
    """
    Штучка, чтобы работать с серверными уведомлениями Qiwi. Запускает сервер для приема уведомлений. А ещё тут есть хендлеры.

    :param auth_key: приватный ключ, на который настроен прием уведомлений.
    :type auth_key: ``str``
    """

    def __init__(self, auth_key: str, once=False):
        self.auth_key = auth_key
        self.handlers = []
        self.once = once
        routes = [Route("/qiwi_notify", endpoint=self.server)]

        self.app = Starlette(routes=routes)

    def handler(self, func=None):
        def decorator(handler):
            self.handlers.append({"handler": handler, "filter": func})
            return handler

        return decorator

    def _check_valid(self, bill: Bill, sha256: str):
        invoice_parameters = f"{bill.currency} | {bill.amount} | {bill.bill_id} | {bill.site_id} | {bill.status}"
        return bool(
            hmac.new(
                self.auth_key.encode(), invoice_parameters.encode(), hashlib.sha256
            ).hexdigest()
            == sha256
        )

    async def _parse(self, response, sha256):
        bill = Bill(response)
        if self._check_valid(bill, sha256):
            loop = asyncio.get_event_loop()
            loop.create_task(self._check(bill))
            return Response(status_code=status.HTTP_200_OK)
        return Response(status_code=status.HTTP_401_UNAUTHORIZED)

    async def _check(self, bill):
        for handler in self.handlers:
            if handler["filter"](bill):
                if inspect.iscoroutinefunction(handler["handler"]):
                    await handler["handler"](bill)
                else:
                    handler["handler"](bill)
                if self.once:
                    break

    async def server(self, request: Request):
        await self._parse(
            await request.json(), request.headers["X-Api-Signature-SHA256"]
        )

    def start(self, port: int = 12345, log_level="info"):
        config = Config()
        config.bind = [f"0.0.0.0:{port}"]
        asyncio.run(serve(self.app, config))

    async def a_start(self, port: int = 12345, log_level="info"):
        config = Config()
        config.bind = [f"0.0.0.0:{port}"]
        await serve(self.app, Config())



import json
import time
import typing

from httpx._models import Response
from response_report import Reporter

from pyqiwip2p.p2p_types import QiwiCustomer, QiwiDatetime, QiwiError


class Bill:
    """
    Объект для удобной работы со счетом

    **Аргументы**

    :param response: ответ от серверов киви. Можно просто json.
    :type response: Response or ``dict``
    :param qiwi_p2p: объект P2P для работы дополнительных методов
    :type qiwi_p2p: QiwiP2P, optional

    **Атрибуты**

    :param site_id: идентификатор вашего сайта в системе Qiwi
    :type site_id: ``str``
    :param bill_id: идентификатор счета
    :type bill_id: ``str``
    :param amount: сумма счета
    :type amount: ``float``
    :param currency: валюта счета
    :type currency: ``str``
    :param status: статус счета
    :type status: ``str``
    :param status_changed: время последнего изменения счёта
    :type status_changed: QiwiDatetime
    :param creation: время создания счета
    :type creation: QiwiDatetime
    :param expiration: время закрытия счета
    :type expiration: QiwiDatetime
    :param pay_url: URL-адрес для оплаты
    :type pay_url: ``str``
    :param comment: комментарий
    :type comment: ``str``, optional
    :param customer: информация о клиенте
    :type customer: QiwiCustomer
    :param fields: кастомные поля Qiwi
    :type fields: ``dict``
    :param json: исходный словарь Qiwi на случай, если они что-то обновят или у меня что-то не работает
    :type json: ``dict``
    :param alt_url: ссылка с проксированием через сервер для установления заголовка referer
    :type alt_url: ``str``

    """

    def __init__(self, response: typing.Union[Response, dict], alt="qp2p.0708.su"):
        self.r_json = response
        if type(self.r_json) is Response:
            try:
                self.r_json = self.r_json.json()
            except json.decoder.JSONDecodeError:
                fn = f"QiwiCrash_{int(time.time())}.txt"
                Reporter(response).save(fn)
                raise ValueError(
                    f"Code: {response.status_code}. "
                    f"Qiwi response is not JSON. This is Qiwi-side bug. "
                    f"Please try again later or check response. "
                    f"Qiwi response saved to {fn}. "
                    f"P.S. The number of requests per minute may have been exceeded. "
                    f"You can wait, change auth_key or cry."
                )
        if "errorCode" in self.r_json:
            fn = f"QiwiCrash_{int(time.time())}.txt"
            Reporter(response).save(fn)
            raise QiwiError(self.r_json)

        self.site_id: int = self.r_json["siteId"]
        self.bill_id: int = self.r_json["billId"]
        self.amount: float = self.r_json["amount"]["value"]
        self.currency: str = self.r_json["amount"]["currency"]
        self.status: str = self.r_json["status"]["value"]
        self.status_changed: str = self.r_json["status"]["changedDateTime"]
        self.creation: str = self.r_json["creationDateTime"]
        self.expiration: str = self.r_json["expirationDateTime"]
        self.pay_url: str = self.r_json["payUrl"]
        self.comment: str = self.r_json["comment"] if "comment" in self.r_json else None
        self.customer: QiwiCustomer = (
            QiwiCustomer(json_data=self.r_json["customer"])
            if "customer" in self.r_json
            else None
        )
        self.fields: dict = (
            self.r_json["customFields"] if "customFields" in self.r_json else None
        )
        self.json: dict = self.r_json
        self.alt_url: str = f"https://{alt}/bill/{self.pay_url[-36:]}"



import re
import typing
from datetime import datetime, timedelta, timezone


class QiwiDatetime:
    """
    Тип для удобной работы с форматами времени.

    **Аргументы**

    :param moment: нужный момент времени в одном из удобных форматов для универсализации
    :type moment: ``str``, ``int``, ``datetime``, optional, default=now
    :param lifetime: время жизни счета. Генерирует момент времени с разницей в ``lifetime`` минут. Если указано, параметр ``moment`` игнорируется.
    :type lifetime: ``int``

    **Атрибуты**

    :param datetime: момент времени
    :type datetime: ``datetime.datetime``
    :param qiwi: момент времени
    :type qiwi: ``str`` в формате "*YYYY-MM-DDThh:mm:ss.mss+hh:mm*"
    :param timestamp: момент времени
    :type timestamp: ``int`` в формате unix-времени
    """

    def __init__(
        self, moment: typing.Union[str, int, datetime] = None, lifetime: int = None
    ):
        if lifetime:
            self.expiration(lifetime)
        else:
            self._exp_regex = r"[0-9]{4}-[01][0-9]-[0-3][0-9]T[0-2][0-9]:[0-5][0-9]:[0-5][0-9].[0-9]{3}[+-][0-1][0-9]:[0-5][0-9]"
            self.datetime: datetime = self.now_datetime()
            self.qiwi = self.datetime.isoformat()
            self.timestamp = self.datetime.timestamp()
            if moment:
                if type(moment) is str:
                    if re.match(self._exp_regex, moment):
                        self.set_from_qiwi(moment)
                    else:
                        raise TypeError(
                            r"The string does not match the format 'ГГГГ-ММ-ДДTчч:мм:сс.мсс+\-чч:мм'"
                        )
                if type(moment) is int or type(moment) is float:
                    self.set_from_timestamp(moment)
                if type(moment) is datetime:
                    self.set_from_datetime(moment)
            else:
                self.set_from_datetime(self.now_datetime())

    def now_datetime(self):
        return datetime.now(timezone(timedelta(hours=3))).replace(microsecond=0)

    def timestamp_datetime(self, timestamp: int):
        return (
            datetime.fromtimestamp(timestamp)
            .astimezone(timezone.utc)
            .replace(microsecond=0)
        )

    def timestamp_qiwi(self, timestamp: int):
        return (
            self.timestamp_datetime(timestamp)
            .astimezone(timezone(timedelta(hours=3)))
            .isoformat()
        )

    def datetime_timastamp(self, dt: datetime):
        return int(dt.timestamp())

    def datetime_qiwi(self, dt: datetime):
        return (
            dt.astimezone(timezone(timedelta(hours=3)))
            .replace(microsecond=0)
            .isoformat()
        )

    def qiwi_timestamp(self, dt: str):
        return int(self.qiwi_datetime(dt).timestamp())

    def qiwi_datetime(self, dt: str):
        return datetime.strptime(dt, "%Y-%m-%dT%H:%M:%S.%f%z")

    def set_from_timestamp(self, timestamp: typing.Union[int, float]):
        self.datetime: datetime = self.timestamp_datetime(int(timestamp))
        self.qiwi = self.timestamp_qiwi(int(timestamp))
        self.timestamp = int(timestamp)

    def set_from_qiwi(self, dt: str):
        self.datetime = self.qiwi_datetime(dt)
        self.qiwi = dt
        self.timestamp = self.qiwi_timestamp(dt)

    def set_from_datetime(self, dt: datetime):
        self.datetime: dt
        self.qiwi = self.datetime_qiwi(dt)
        self.timestamp = self.datetime_timastamp(dt)

    def expiration(self, lifetime: int = 30):
        """
        Рассчитывает время, когда надо закрыть счет

        :param lifetime: через сколько минут закрыть счет
        :type lifetime: ``int``, optional, default=30
        :return: Время закрытия заказа
        :rtype: ``str`` в формате "*YYYY-MM-DDThh:mm:ss+hh:mm*"
        """
        self.set_from_datetime(self.now_datetime() + timedelta(minutes=lifetime))



from pyqiwip2p.p2p_types.qiwi_datetime import *
from pyqiwip2p.p2p_types.custom_fields import *
from pyqiwip2p.p2p_types.customer import *
from pyqiwip2p.p2p_types.errors import *
from pyqiwip2p.p2p_types.responses import *



from pyqiwip2p.p2p_types import QiwiDatetime


class QiwiError(Exception):
    """
    Штучка для удобной работы с ошибками от Qiwi

    **Аргументы**

    :param response_json: json-словарь ответа на запрос
    :type response_json: ``dict``

    **Атрибуты**

    :param service_name: название сервсиса
    :type service_name: ``str``
    :param error_code: код ошибки
    :type error_code: ``str``
    :param description: описание ошибки
    :type description: ``str``
    :param user_message: сообщение
    :type description: ``str``
    :param datetime: момент времени
    :type datetime: QiwiDatetime
    :param trace_id: идентификатор ошибки
    :type trace_id: ``str``
    :param cause: поясление
    :type cause: ``str``, optional
    """

    def __init__(self, response_json: dict):
        self.service_name = response_json["serviceName"]
        self.error_code = response_json["errorCode"]
        self.description = response_json["description"]
        self.user_message = response_json["userMessage"]
        self.datetime = QiwiDatetime(response_json["dateTime"])
        self.trace_id = response_json["traceId"]
        self.cause = response_json["cause"] if "cause" in response_json else None
        if self.cause:
            message = f"{self.error_code} - {self.description}. {self.cause}"
        else:
            message = f"{self.error_code} - {self.description}."
        super().__init__(message)

    def __str__(self):
        return f"{self.error_code} - {self.description}"



import hashlib
import hmac
import json
import threading

import cherrypy

from pyqiwip2p.notify.async_client import AioQiwiNotify
from pyqiwip2p.p2p_types import Bill


class QiwiNotify:
    """
    Штучка, чтобы работать с серверными уведомлениями Qiwi. Запускает сервер для приема уведомлений. А ещё тут есть хендлеры.

    :param auth_key: приватный ключ, на который настроен прием уведомлений.
    :type auth_key: ``str``
    """

    def __init__(self, auth_key: str, once=True):
        self.auth_key = auth_key
        self.handlers = []
        self.once = once

    def handler(self, func=None):
        def decorator(handler):
            self.handlers.append({"handler": handler, "filter": func})
            return handler

        return decorator

    def _check_valid(self, bill: Bill, sha256: str):
        invoice_parameters = f"{bill.currency} | {bill.amount} | {bill.bill_id} | {bill.site_id} | {bill.status}"
        return (
            hmac.new(
                self.auth_key.encode(), invoice_parameters.encode(), hashlib.sha256
            ).hexdigest()
            == sha256
        )

    def _parse(self, response, sha256):
        bill = Bill(response)
        if self._check_valid(bill, sha256):
            self._check(bill)
        else:
            raise cherrypy.HTTPError(403)

    def _check(self, bill):
        for handler in self.handlers:
            if handler["filter"](bill):
                handler["handler"](bill)
            if self.once:
                break

    def start(qiwi, port: int = 8099):
        """
        Функция для запуска веб-сервера, который будет обрабатывать входящие запросы и запускать захендленные функции.
        Внимание, предполагается проксирование, так как Qiwi отправляет запросы на
        443 порт серверов с шифрованием доверенным сертификатом (SSL). Подниманиемый сервер не защищен никаким
        сертификатом и вообще ничего не может сделать для Qiwi. Рекомендую настроить Nginx.

        :param port: номер порта, на котором запустится сервер.
        :type port: ``int``
        """

        class WebhookServer(object):
            def check_headers(self, headers):
                if (
                    "content-type" in headers
                    and 'application/json' in headers["content-type"]
                    and "X-Api-Signature-SHA256" in headers
                ):
                    return True
                return False

            @cherrypy.expose
            def qiwi_notify(self):
                if self.check_headers(cherrypy.request.headers):
                    json_string = cherrypy.request.body.read().decode("utf-8")
                    qiwi._parse(
                        json.loads(json_string),
                        cherrypy.request.headers["X-Api-Signature-SHA256"],
                    )
                    return ""
                raise cherrypy.HTTPError(403)

        cherrypy.config.update({"server.socket_port": port})
        t = threading.Thread(
            target=lambda ws: cherrypy.quickstart(ws()), args=(WebhookServer,)
        )
        t.start()



import json
import random
import time
import typing
from base64 import b64decode
from ipaddress import IPv4Address, IPv4Network

import httpx
from loguru import logger

from pyqiwip2p.p2p_types import Bill, PaymentMethods, QiwiCustomer, QiwiDatetime

logger.disable("pyqiwip2p")


class QiwiP2P:
    """
    Основной инструмент-клиент для взаимодействия с API QiwiP2P

    **Аргументы и атрибуты**

    :param auth_key: приватный ключ авторизации со страницы https://qiwi.com/p2p-admin/transfers/api. Нужен для работы с вашим аккаунтом.
    :type auth_key: ``str``
    :param default_amount: значение суммы счета по умолчанию для новых счетов.
    :type default_amount: ``int`` or ``float``, optional
    :param currency: валюта для счетов в формате *Alpha-3 ISO 4217*. Пока что API умеет работать только с *RUB* и *KZT*
    :type currency: ``str``, optional
    :param alt: альтернативный домен для проксирующей ссылки. По-умолчанию ``qp2p.0708.su``
    :type alt: ``str``, optional
    """

    def __init__(
        self,
        auth_key: str,
        default_amount: int = 100,
        currency: str = "RUB",
        alt: str = "qp2p.0708.su",
        skip_key_validation: bool = False,
    ):
        if not skip_key_validation:
            self.validate_privkey(auth_key)
        self.validate_privkey(auth_key)
        self.auth_key = auth_key
        self.default_amount = default_amount
        self.is_async = False
        self._client: httpx.Client = None
        self.alt = alt
        if currency not in ["RUB", "KZT"]:
            raise ValueError('Currency must be "RUB" or "KZT"')
        self.currency = currency

    @property
    def client(self):
        if not self._client:
            self._client = httpx.Client()
        return self._client

    def __enter__(self):
        return self

    def __exit__(self, *args, **kwargs):
        return self.client.close()

    @staticmethod
    def is_qiwi_ip(ip: str, qiwi_ips=None, *args, **kwargs):
        """
        Вы просили, мы сделали. Теперь проверить IP можнно одной простой функцией.
        Причём не обязательно инициализировать объект, так как это статичный метод.

        :param ip: ip адрес, с которого пришёл запрос
        :type ip: ``str``
        :param qiwi_ips: список разрешённых подсетей
        :type qiwi_ips: ``list`` or ``tuple``, optional
        :return: Принадлежность IP адресам Qiwi
        :rtype: ``bool``
        """
        if qiwi_ips is None:
            qiwi_ips = [
                "79.142.16.0/20",
                "195.189.100.0/22",
                "91.232.230.0/23",
                "91.213.51.0/24",
            ]
        ip = IPv4Address(ip)
        is_qiwi = any(ip in IPv4Network(net) for net in qiwi_ips)
        logger.log(
            "debug" if is_qiwi else "warning",
            f"is_qiwi_ip: {ip} {'not ' if not is_qiwi else ''} in {qiwi_ips}",
        )
        return is_qiwi

    def validate_privkey(self, privkey):
        key_decoded = b64decode(privkey).decode()
        try:
            key_decoded = json.loads(key_decoded)
            if "version" in key_decoded and "data" in key_decoded:
                key_data = key_decoded["data"]
                if (
                    "payin_merchant_site_uid" in key_data
                    and "user_id" in key_data
                    and "secret" in key_data
                ):
                    if key_decoded["version"] == "P2P":
                        return True
        except json.decoder.JSONDecodeError:
            ...
        raise ValueError("Invalid token")

    def bill(
        self,
        bill_id: typing.Union[str, int] = None,
        amount: typing.Union[int, float] = None,
        currency: str = None,
        expiration: typing.Union[str, int, QiwiDatetime] = None,
        lifetime: int = 30,
        customer: typing.Union[QiwiCustomer, dict] = None,
        comment: str = "via pyQiwiP2P (WhiteApfel)",
        pay_sources: list[str] = None,
        theme_code: str = None,
        fields: dict = None,
    ) -> Bill:
        """
        Метод для выставления счета.

        :param bill_id: идентификатор заказа/счета в вашей системе. Если параметр не укзаан, генерируется строка, основанная на времени.
        :type bill_id: ``str`` or ``int``, optional
        :param amount: сумма заказа в рублях. Округляется до двух знаков после запятой. Если не указано, используется значение по умолчанию
        :type amount: ``int`` or ``float``, optional
        :param currency: валюта платежа. ``RUB`` - рубли,  ``KZT`` - тенге
        :type currency: ``str`` or None, optional
        :param expiration: когда счет будет закрыт. Принимает: Timestamp, Datetime или строку формата ``YYYY-MM-DDThh:mm:ss+hh:mm``.
        :type expiration: ``int``, ``datetime`` or ``str``, optional
        :param lifetime: время жизни счета в минутах. Если параметр ``expiration`` не указан, то будет автоматически сгенерируется дата для закрытия через ``lifetime`` минут.
        :type lifetime: ``int``, optional, default=30
        :param customer: объект QiwiCustomer или ``dict`` с полями phone, email и customer
        :type customer: ``QiwiCustomer`` or ``dict``, optional
        :param comment: комментарий к платежу. До 255 символов
        :type comment: ``str``, optional
        :param pay_sources: лист методов оплаты, которые хотим разрешить. Методы есть в классе PaymentMethods
        :type pay_sources: ``list[str]``, optional
        :param theme_code: код темы оформления, можно получить на сайте киви
        :type theme_code: ``str``, optional
        :param fields: словарь кастомных полей QIWI. Я ничего про них не понял, извините. Параметры pay_sources и theme_code перезаписывают соответствующие поля в fields
        :type fields: ``dict``, optional
        :raise QiwiError: объект ответа Qiwi, если запрос не увенчался успехом
        :return: Объект счета при успешном выполнении
        :rtype: Bill
        """
        logger.info(
            f"bill args: bill_id: {bill_id}, amount: {amount}, currency: {currency}, "
            f"expiration: {expiration}, "
            f"lifetime: {lifetime}, customer: {customer}, fields: {fields}"
        )
        fields = fields or {}
        bill_id = (
            bill_id
            or f"WhiteApfel-PyQiwiP2P-{str(int(time.time() * 100))[4:]}-"
            f"{int(random.random() * 1000)}"
        )

        amount = amount or self.default_amount
        amount_round = str(round(float(amount), 2))
        amount = (
            amount_round
            if len(str(float(amount)).split(".")[1]) > 1
            else str(round(float(amount), 2)) + "0"
        )

        expiration = (
            QiwiDatetime(moment=expiration).qiwi
            if expiration
            else QiwiDatetime(lifetime=lifetime).qiwi
        )

        if currency and currency not in ["RUB", "KZT"]:
            raise ValueError(f'Currency must be "RUB" or "KZT", not "{currency}"')
        currency = self.currency

        if pay_sources is not None:
            allowed_sources = [
                PaymentMethods.qiwi,
                PaymentMethods.card,
                PaymentMethods.mobile,
            ]
            if isinstance(pay_sources, list) and all(
                i in allowed_sources for i in pay_sources
            ):
                fields["paySourcesFilter"] = ",".join(pay_sources)
            else:
                logger.warning(
                    "pay_sources must be list of str from PaymentMethods ('qw', 'card', 'mobile')"
                )

        if theme_code is not None:
            fields["themeCode"] = theme_code

        qiwi_request_headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.auth_key}",
        }

        qiwi_request_data = {
            "amount": {"currency": currency, "value": amount},
            "comment": comment,
            "expirationDateTime": expiration,
            "customer": customer.dict
            if type(customer) is QiwiCustomer
            else QiwiCustomer(json_data=customer).dict
            if customer
            else {},
            "customFields": fields,
        }
        logger.info(
            f"bill request: bill_id: {bill_id}, amount: {amount}, "
            f"currency: {currency}, expiration: "
            f"{expiration}, lifetime: {lifetime}, "
            f"customer: {customer}, fields: {fields}"
        )
        qiwi_raw_response = self.client.put(
            f"https://api.qiwi.com/partner/bill/v1/bills/{bill_id}",
            json=qiwi_request_data,
            headers=qiwi_request_headers,
        )
        qiwi_response = Bill(qiwi_raw_response, self.alt)
        logger.info(f"bill created: pay_url: {qiwi_response.pay_url}")
        return qiwi_response

    def check(self, bill_id: typing.Union[str, int, Bill]) -> Bill:
        """
        Проверяет статус выставленного счета.

        :param bill_id: идентификатор заказа/счета в вашей системе
        :type bill_id: ``str`` or ``int``
        :return: Объект счета при успешном выполнении
        :rtype: Bill
        """
        logger.info(f"check bill: bill_id: {bill_id}")
        if type(bill_id) is Bill:
            bill_id = bill_id.bill_id
        qiwi_request_headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.auth_key}",
        }

        qiwi_raw_response = self.client.get(
            f"https://api.qiwi.com/partner/bill/v1/bills/{bill_id}",
            headers=qiwi_request_headers,
        )
        qiwi_response = Bill(qiwi_raw_response, self.alt)
        logger.info(f"checked bill: bill_id: {bill_id}, status: {qiwi_response.status}")
        return qiwi_response

    def reject(self, bill_id: typing.Union[str, int, Bill]) -> Bill:
        """
        Закрывает счет на оплату.

        :param bill_id: идентификатор заказа/счета в вашей системе
        :type bill_id: ``str`` or ``int``
        :return: Объект счета при успешном выполнении
        :rtype: Bill
        """
        logger.info(f"reject bill: bill_id: {bill_id}")
        if type(bill_id) is Bill:
            bill_id = bill_id.bill_id
        qiwi_request_headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.auth_key}",
        }
        qiwi_raw_response = self.client.post(
            f"https://api.qiwi.com/partner/bill/v1/bills/{bill_id}/reject",
            headers=qiwi_request_headers,
        )
        qiwi_response = Bill(qiwi_raw_response, self.alt)
        logger.info(
            f"rejected bill: bill_id: {bill_id}, status: {qiwi_response.status}"
        )
        return qiwi_response
